@@ -62,9 +62,6 @@ char cm_mountRoot[1024];
 DWORD cm_mountRootLen;
 int cm_logChunkSize;
 int cm_chunkSize;
-#ifdef AFS_FREELANCE_CLIENT
-char *cm_FakeRootDir;
-#endif /* freelance */
 
 int smb_UseV3 = 1;
 
@@ -558,9 +555,11 @@ int afsd_InitCM(char **reasonP)
     DWORD blockSize;
     long logChunkSize;
     DWORD stats;
+    DWORD volumes;
+    DWORD cells;
     DWORD dwValue;
-    DWORD rx_enable_peer_stats = 0;
-    DWORD rx_enable_process_stats = 0;
+    DWORD rx_enable_peer_stats;
+    DWORD rx_enable_process_stats;
     long traceBufSize;
     long maxcpus;
     long ltt, ltto;
@@ -776,10 +775,30 @@ int afsd_InitCM(char **reasonP)
     code = RegQueryValueEx(parmKey, "Stats", NULL, NULL,
                             (BYTE *) &stats, &dummyLen);
     if (code == ERROR_SUCCESS)
-        afsi_log("Status cache size %d", stats);
+        afsi_log("Status cache entries: %d", stats);
     else {
         stats = CM_CONFIGDEFAULT_STATS;
-        afsi_log("Default status cache size %d", stats);
+        afsi_log("Default status cache entries: %d", stats);
+    }
+
+    dummyLen = sizeof(volumes);
+    code = RegQueryValueEx(parmKey, "Volumes", NULL, NULL,
+                            (BYTE *) &volumes, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        afsi_log("Volumes cache entries: %d", volumes);
+    else {
+        volumes = CM_CONFIGDEFAULT_STATS / 3;
+        afsi_log("Default volume cache entries: %d", volumes);
+    }
+
+    dummyLen = sizeof(cells);
+    code = RegQueryValueEx(parmKey, "Cells", NULL, NULL,
+                            (BYTE *) &cells, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        afsi_log("Cell cache entries: %d", cells);
+    else {
+        cells = CM_CONFIGDEFAULT_CELLS;
+        afsi_log("Default cell cache entries: %d", cells);
     }
 
     dummyLen = sizeof(ltt);
@@ -1044,19 +1063,23 @@ int afsd_InitCM(char **reasonP)
     code = RegQueryValueEx(parmKey, "RxEnablePeerStats", NULL, NULL,
                            (BYTE *) &rx_enable_peer_stats, &dummyLen);
     if (code != ERROR_SUCCESS) {
-        rx_enable_peer_stats = 0;
+        rx_enable_peer_stats = 1;
     }
     if (rx_enable_peer_stats)
         afsi_log("RX Peer Statistics gathering is enabled");
+    else
+        afsi_log("RX Peer Statistics gathering is disabled");
 
     dummyLen = sizeof(rx_enable_process_stats);
     code = RegQueryValueEx(parmKey, "RxEnableProcessStats", NULL, NULL,
                            (BYTE *) &rx_enable_process_stats, &dummyLen);
     if (code != ERROR_SUCCESS) {
-        rx_enable_process_stats = 0;
+        rx_enable_process_stats = 1;
     }
     if (rx_enable_process_stats)
         afsi_log("RX Process Statistics gathering is enabled");
+    else
+        afsi_log("RX Process Statistics gathering is disabled");
 
     dummyLen = sizeof(dwValue);
     dwValue = 0;
@@ -1199,7 +1222,7 @@ int afsd_InitCM(char **reasonP)
     cm_initParams.nChunkFiles = 0;
     cm_initParams.nStatCaches = stats;
     cm_initParams.nDataCaches = (afs_uint32)(cacheBlocks > 0xFFFFFFFF ? 0xFFFFFFFF : cacheBlocks);
-    cm_initParams.nVolumeCaches = stats/2;
+    cm_initParams.nVolumeCaches = volumes;
     cm_initParams.firstChunkSize = cm_chunkSize;
     cm_initParams.otherChunkSize = cm_chunkSize;
     cm_initParams.cacheSize = cacheSize;
@@ -1222,7 +1245,7 @@ int afsd_InitCM(char **reasonP)
         
     cm_InitCallback();
 
-    code = cm_InitMappedMemory(virtualCache, cm_CachePath, stats, cm_chunkSize, cacheBlocks, blockSize);
+    code = cm_InitMappedMemory(virtualCache, cm_CachePath, stats, volumes, cells, cm_chunkSize, cacheBlocks, blockSize);
     afsi_log("cm_InitMappedMemory code %x", code);
     if (code != 0) {
         *reasonP = "error initializing cache file";
@@ -1348,9 +1371,9 @@ int afsd_InitDaemons(char **reasonP)
 
         osi_Log0(afsd_logp, "Loading Root Volume from cell");
 	do {
-	    code = cm_GetVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
+	    code = cm_FindVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
 				       &req, CM_GETVOL_FLAG_CREATE, &cm_data.rootVolumep);
-	    afsi_log("cm_GetVolumeByName code %x root vol %x", code,
+	    afsi_log("cm_FindVolumeByName code %x root vol %x", code,
 		      (code ? (cm_volume_t *)-1 : cm_data.rootVolumep));
 	} while (code && --attempts);
         if (code != 0) {
@@ -1401,7 +1424,7 @@ int afsd_InitSMB(char **reasonP, void *aMBfunc)
         code = RegQueryValueEx(parmKey, "EnableSMBAsyncStore", NULL, NULL,
                                 (BYTE *) &dwValue, &dummyLen);
         if (code == ERROR_SUCCESS)
-            smb_AsyncStore = dwValue ? 1 : 0;
+            smb_AsyncStore = dwValue == 2 ? 2 : (dwValue ? 1 : 0);
         afsi_log("EnableSMBAsyncStore = %d", smb_AsyncStore);
 
         dummyLen = sizeof(DWORD);
