@@ -130,10 +130,8 @@ void buf_ReleaseLocked(cm_buf_t *bp, afs_uint32 writeLocked)
          * double check that the refCount is actually zero
          * before we remove the buffer from the LRU queue.
          */
-        if (!writeLocked) {
-            lock_ReleaseRead(&buf_globalLock);
-            lock_ObtainWrite(&buf_globalLock);
-        }
+        if (!writeLocked)
+            lock_ConvertRToW(&buf_globalLock);
 
         if (bp->refCount == 0 &&
             !(bp->flags & CM_BUF_INLRU)) {
@@ -145,10 +143,8 @@ void buf_ReleaseLocked(cm_buf_t *bp, afs_uint32 writeLocked)
             bp->flags |= CM_BUF_INLRU;
         }
 
-        if (!writeLocked) {
-            lock_ReleaseWrite(&buf_globalLock);
-            lock_ObtainRead(&buf_globalLock);
-        }
+        if (!writeLocked)
+            lock_ConvertWToR(&buf_globalLock);
     }
 }       
 
@@ -204,11 +200,10 @@ void buf_IncrSyncer(long parm)
 
         /* now go through our percentage of the buffers */
         for (bpp = &cm_data.buf_dirtyListp; bp = *bpp; ) {
-
 	    /* all dirty buffers are held when they are added to the
 	     * dirty list.  No need for an additional hold.
 	     */
-
+            lock_ObtainMutex(&bp->mx);
 	    if (bp->flags & CM_BUF_DIRTY) {
 		/* start cleaning the buffer; don't touch log pages since
  		 * the log code counts on knowing exactly who is writing
@@ -216,33 +211,27 @@ void buf_IncrSyncer(long parm)
 		 */
 		cm_InitReq(&req);
 		req.flags |= CM_REQ_NORETRY;
-		wasDirty |= buf_CleanAsync(bp, &req);
+		wasDirty |= buf_CleanAsyncLocked(bp, &req);
 	    }
 
 	    /* the buffer may or may not have been dirty
 	     * and if dirty may or may not have been cleaned
 	     * successfully.  check the dirty flag again.  
 	     */
-	    if (!(bp->flags & CM_BUF_DIRTY)) {
-		lock_ObtainMutex(&bp->mx);
-		if (!(bp->flags & CM_BUF_DIRTY)) {
-		    /* remove the buffer from the dirty list */
-		    lock_ObtainWrite(&buf_globalLock);
-		    *bpp = bp->dirtyp;
-		    bp->dirtyp = NULL;
-		    if (cm_data.buf_dirtyListp == NULL)
-			cm_data.buf_dirtyListEndp = NULL;
-		    buf_ReleaseLocked(bp, TRUE);
-		    lock_ReleaseWrite(&buf_globalLock);
-		} else {
-		    /* advance the pointer so we don't loop forever */
-		    bpp = &bp->dirtyp;
-		}
-		lock_ReleaseMutex(&bp->mx);
-	    } else {
-		/* advance the pointer so we don't loop forever */
-		bpp = &bp->dirtyp;
-	    }
+            if (!(bp->flags & CM_BUF_DIRTY)) {
+                /* remove the buffer from the dirty list */
+                lock_ObtainWrite(&buf_globalLock);
+                *bpp = bp->dirtyp;
+                bp->dirtyp = NULL;
+                if (cm_data.buf_dirtyListp == NULL)
+                    cm_data.buf_dirtyListEndp = NULL;
+                buf_ReleaseLocked(bp, TRUE);
+                lock_ReleaseWrite(&buf_globalLock);
+            } else {
+                /* advance the pointer so we don't loop forever */
+                bpp = &bp->dirtyp;
+            }
+            lock_ReleaseMutex(&bp->mx);
         }	/* for loop over a bunch of buffers */
     }		/* whole daemon's while loop */
 }
@@ -571,12 +560,12 @@ void buf_WaitIO(cm_scache_t * scp, cm_buf_t *bp)
 		 release = 1;
         }
         if ( scp ) {
-            lock_ObtainMutex(&scp->mx);
+            lock_ObtainRead(&scp->rw);
             if (scp->flags & CM_SCACHEFLAG_WAITING) {
                 osi_Log1(buf_logp, "buf_WaitIO waking scp 0x%p", scp);
                 osi_Wakeup((LONG_PTR)&scp->flags);
             }
-	    lock_ReleaseMutex(&scp->mx);
+	    lock_ReleaseRead(&scp->rw);
         }
     }
         
@@ -657,7 +646,16 @@ long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
 
             offset = bp->offset;
             LargeIntegerAdd(offset, ConvertLongToLargeInteger(bp->dirty_offset));
-	    code = (*cm_buf_opsp->Writep)(scp, &offset, bp->dirty_length, 0, bp->userp, reqp);
+	    code = (*cm_buf_opsp->Writep)(scp, &offset, 
+#if 1
+                                           /* we might as well try to write all of the contiguous 
+                                            * dirty buffers in one RPC 
+                                            */
+                                           cm_chunkSize,
+#else
+                                          bp->dirty_length, 
+#endif
+                                          0, bp->userp, reqp);
 	    osi_Log3(buf_logp, "buf_CleanAsyncLocked I/O on scp 0x%p buf 0x%p, done=%d", scp, bp, code);
 
 	    cm_ReleaseSCache(scp);
@@ -678,7 +676,7 @@ long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
             bp->dirty_offset = 0;
             bp->dirty_length = 0;
 	    bp->error = code;
-	    bp->dataVersion = -1; /* bad */
+	    bp->dataVersion = CM_BUF_VERSION_BAD; /* bad */
 	    bp->dirtyCounter++;
 	}
 
@@ -775,12 +773,6 @@ void buf_Recycle(cm_buf_t *bp)
         bp->flags &= ~CM_BUF_INHASH;
     }
 
-    /* bump the soft reference counter now, to invalidate softRefs; no
-     * wakeup is required since people don't sleep waiting for this
-     * counter to change.
-     */
-    bp->idCounter++;
-
     /* make the fid unrecognizable */
     memset(&bp->fid, 0, sizeof(cm_fid_t));
 }       
@@ -914,7 +906,7 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
 
             /* clean up junk flags */
             bp->flags &= ~(CM_BUF_EOF | CM_BUF_ERROR);
-            bp->dataVersion = -1;	/* unknown so far */
+            bp->dataVersion = CM_BUF_VERSION_BAD;	/* unknown so far */
 
             /* now hash in as our new buffer, and give it the
              * appropriate label, if requested.
@@ -1196,7 +1188,7 @@ long buf_CountFreeList(void)
          * has been invalidate (by having its DV stomped upon), then
          * count it as free, since it isn't really being utilized.
          */
-        if (!(bufp->flags & CM_BUF_INHASH) || bufp->dataVersion <= 0)
+        if (!(bufp->flags & CM_BUF_INHASH) || bufp->dataVersion == CM_BUF_VERSION_BAD)
             count++;
     }       
     lock_ReleaseRead(&buf_globalLock);
@@ -1217,15 +1209,17 @@ long buf_CleanAsync(cm_buf_t *bp, cm_req_t *reqp)
 }       
 
 /* wait for a buffer's cleaning to finish */
-void buf_CleanWait(cm_scache_t * scp, cm_buf_t *bp)
+void buf_CleanWait(cm_scache_t * scp, cm_buf_t *bp, afs_uint32 locked)
 {
     osi_assertx(bp->magic == CM_BUF_MAGIC, "invalid cm_buf_t magic");
 
-    lock_ObtainMutex(&bp->mx);
+    if (!locked)
+        lock_ObtainMutex(&bp->mx);
     if (bp->flags & CM_BUF_WRITING) {
         buf_WaitIO(scp, bp);
     }
-    lock_ReleaseMutex(&bp->mx);
+    if (!locked)
+        lock_ReleaseMutex(&bp->mx);
 }       
 
 /* set the dirty flag on a buffer, and set associated write-ahead log,
@@ -1331,7 +1325,7 @@ long buf_CleanAndReset(void)
 		req.flags |= CM_REQ_NORETRY;
 
 		buf_CleanAsync(bp, &req);
-		buf_CleanWait(NULL, bp);
+		buf_CleanWait(NULL, bp, FALSE);
 
                 /* relock and release buffer */
                 lock_ObtainRead(&buf_globalLock);
@@ -1442,7 +1436,7 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
              LargeIntegerLessThan(*sizep, bufEnd)) {
             buf_WaitIO(scp, bufp);
         }
-        lock_ObtainMutex(&scp->mx);
+        lock_ObtainWrite(&scp->rw);
 	
         /* make sure we have a callback (so we have the right value for
          * the length), and wait for it to be safe to do a truncate.
@@ -1472,7 +1466,7 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
                 bufp->flags &= ~CM_BUF_DIRTY;
                 bufp->dirty_offset = 0;
                 bufp->dirty_length = 0;
-                bufp->dataVersion = -1;	/* known bad */
+                bufp->dataVersion = CM_BUF_VERSION_BAD;	/* known bad */
                 bufp->dirtyCounter++;
             }
             else {
@@ -1495,7 +1489,7 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
 		       CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS
 		       | CM_SCACHESYNC_SETSIZE | CM_SCACHESYNC_BUFLOCKED);
 
-        lock_ReleaseMutex(&scp->mx);
+        lock_ReleaseWrite(&scp->rw);
         lock_ReleaseMutex(&bufp->mx);
     
 	if (!code) {
@@ -1552,19 +1546,21 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
             if (code && code != CM_ERROR_BADFD) 
                 goto skip;
 
-	    /* if the scp's FID is bad its because we received VNOVNODE 
-	     * when attempting to FetchStatus before the write.  This
-	     * page therefore contains data that can no longer be stored.
-	     */
-	    lock_ObtainMutex(&bp->mx);
-	    bp->flags &= ~CM_BUF_DIRTY;
-	    bp->flags |= CM_BUF_ERROR;
-	    bp->error = code;
-            bp->dirty_offset = 0;
-            bp->dirty_length = 0;
-            bp->dataVersion = -1;	/* known bad */
-            bp->dirtyCounter++;
-	    lock_ReleaseMutex(&bp->mx);
+            if (code == CM_ERROR_BADFD) {
+                /* if the scp's FID is bad its because we received VNOVNODE 
+                 * when attempting to FetchStatus before the write.  This
+                 * page therefore contains data that can no longer be stored.
+                 */
+                lock_ObtainMutex(&bp->mx);
+                bp->flags &= ~CM_BUF_DIRTY;
+                bp->flags |= CM_BUF_ERROR;
+                bp->error = CM_ERROR_BADFD;
+                bp->dirty_offset = 0;
+                bp->dirty_length = 0;
+                bp->dataVersion = CM_BUF_VERSION_BAD;	/* known bad */
+                bp->dirtyCounter++;
+                lock_ReleaseMutex(&bp->mx);
+            }
 
             /* actually, we only know that buffer is clean if ref
              * count is 1, since we don't have buffer itself locked.
@@ -1582,7 +1578,7 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
                 lock_ReleaseWrite(&buf_globalLock);
             }
 
-	    if (code != CM_ERROR_BADFD)
+	    if (code == 0)
 		(*cm_buf_opsp->Unstabilizep)(scp, userp);
         }
 
@@ -1605,14 +1601,14 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     return code;
 }       
 
-/* Must be called with scp->mx held */
+/* Must be called with scp->rw held */
 long buf_ForceDataVersion(cm_scache_t * scp, afs_uint64 fromVersion, afs_uint64 toVersion)
 {
     cm_buf_t * bp;
     afs_uint32 i;
     int found = 0;
 
-    lock_AssertMutex(&scp->mx);
+    lock_AssertAny(&scp->rw);
 
     i = BUF_FILEHASH(&scp->fid);
 
@@ -1652,21 +1648,21 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
     for (; bp; bp = nbp) {
         /* clean buffer synchronously */
         if (cm_FidCmp(&bp->fid, &scp->fid) == 0) {
-            if (userp) {
-                cm_HoldUser(userp);
-                lock_ObtainMutex(&bp->mx);
-                if (bp->userp) 
-                    cm_ReleaseUser(bp->userp);
-                bp->userp = userp;
-                lock_ReleaseMutex(&bp->mx);
-            }   
-            wasDirty = buf_CleanAsync(bp, reqp);
-	    buf_CleanWait(scp, bp);
             lock_ObtainMutex(&bp->mx);
-            if (bp->flags & CM_BUF_ERROR) {
-		code = bp->error;
-                if (code == 0) 
-                    code = -1;
+            if (bp->flags & CM_BUF_DIRTY) {
+                if (userp) {
+                    cm_HoldUser(userp);
+                    if (bp->userp) 
+                        cm_ReleaseUser(bp->userp);
+                    bp->userp = userp;
+                }   
+                wasDirty = buf_CleanAsyncLocked(bp, reqp);
+                buf_CleanWait(scp, bp, TRUE);
+                if (bp->flags & CM_BUF_ERROR) {
+                    code = bp->error;
+                    if (code == 0) 
+                        code = -1;
+                }
             }
             lock_ReleaseMutex(&bp->mx);
         }
@@ -1853,7 +1849,7 @@ long buf_CleanDirtyBuffers(cm_scache_t *scp)
             bp->dirty_length = 0;
 	    bp->flags |= CM_BUF_ERROR;
 	    bp->error = VNOVNODE;
-	    bp->dataVersion = -1; /* bad */
+	    bp->dataVersion = CM_BUF_VERSION_BAD; /* bad */
 	    bp->dirtyCounter++;
 	    if (bp->flags & CM_BUF_WAITING) {
 		osi_Log2(buf_logp, "BUF CleanDirtyBuffers Waking [scp 0x%x] bp 0x%x", scp, bp);
