@@ -528,6 +528,32 @@ long cm_UpdateVolume(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *reqp,
         rwNewstate = rwServers_alldown ? vl_alldown : vl_online;
         roNewstate = roServers_alldown ? vl_alldown : vl_online;
         bkNewstate = bkServers_alldown ? vl_alldown : vl_online;
+    } else if (code == CM_ERROR_NOSUCHVOLUME || code == VL_NOENT) {
+        /* this volume does not exist - we should discard it */
+        if (volp->flags & CM_VOLUMEFLAG_IN_HASH)
+            cm_RemoveVolumeFromNameHashTable(volp);
+        if (volp->rw.flags & CM_VOLUMEFLAG_IN_HASH)
+            cm_RemoveVolumeFromIDHashTable(volp, RWVOL);
+        if (volp->ro.flags & CM_VOLUMEFLAG_IN_HASH)
+            cm_RemoveVolumeFromIDHashTable(volp, ROVOL);
+        if (volp->bk.flags & CM_VOLUMEFLAG_IN_HASH)
+            cm_RemoveVolumeFromIDHashTable(volp, BACKVOL);
+
+        /* Move to the end so it will be recycled first */
+        cm_MoveVolumeToLRULast(volp);
+
+        if (volp->rw.ID)
+            cm_VolumeStatusNotification(volp, volp->rw.ID, volp->rw.state, vl_alldown);
+        if (volp->ro.ID)
+            cm_VolumeStatusNotification(volp, volp->ro.ID, volp->ro.state, vl_alldown);
+        if (volp->bk.ID)
+            cm_VolumeStatusNotification(volp, volp->bk.ID, volp->bk.state, vl_alldown);
+
+        volp->rw.ID = volp->ro.ID = volp->bk.ID = 0;
+        cm_SetFid(&volp->rw.dotdotFid, 0, 0, 0, 0);
+        cm_SetFid(&volp->ro.dotdotFid, 0, 0, 0, 0);
+        cm_SetFid(&volp->bk.dotdotFid, 0, 0, 0, 0);
+        volp->namep[0] ='\0';
     } else {
         rwNewstate = roNewstate = bkNewstate = vl_alldown;
     }
@@ -551,17 +577,51 @@ long cm_UpdateVolume(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *reqp,
     return code;
 }
 
+/* Requires read or write lock on cm_volumeLock */
 void cm_GetVolume(cm_volume_t *volp)
 {
-    if (volp) {
-	lock_ObtainWrite(&cm_volumeLock);
-	volp->refCount++;
-	lock_ReleaseWrite(&cm_volumeLock);
-    }
+    InterlockedIncrement(&volp->refCount);
 }
 
+cm_volume_t *cm_GetVolumeByFID(cm_fid_t *fidp)
+{
+    cm_volume_t *volp;
+    afs_uint32 hash;
 
-long cm_GetVolumeByID(cm_cell_t *cellp, afs_uint32 volumeID, cm_user_t *userp,
+    lock_ObtainRead(&cm_volumeLock);
+    hash = CM_VOLUME_ID_HASH(fidp->volume);
+    /* The volumeID can be any one of the three types.  So we must
+     * search the hash table for all three types until we find it.
+     * We will search in the order of RO, RW, BK.
+     */
+    for ( volp = cm_data.volumeROIDHashTablep[hash]; volp; volp = volp->ro.nextp) {
+        if ( fidp->cell == volp->cellp->cellID && fidp->volume == volp->ro.ID )
+            break;
+    }
+    if (!volp) {
+        /* try RW volumes */
+        for ( volp = cm_data.volumeRWIDHashTablep[hash]; volp; volp = volp->rw.nextp) {
+            if ( fidp->cell == volp->cellp->cellID && fidp->volume == volp->rw.ID )
+                break;
+        }
+    }
+    if (!volp) {
+        /* try BK volumes */
+        for ( volp = cm_data.volumeBKIDHashTablep[hash]; volp; volp = volp->bk.nextp) {
+            if ( fidp->cell == volp->cellp->cellID && fidp->volume == volp->bk.ID )
+                break;
+        }
+    }
+
+    /* hold the volume if we found it */
+    if (volp) 
+        cm_GetVolume(volp);
+        
+    lock_ReleaseRead(&cm_volumeLock);
+    return volp;
+}
+
+long cm_FindVolumeByID(cm_cell_t *cellp, afs_uint32 volumeID, cm_user_t *userp,
                       cm_req_t *reqp, afs_uint32 flags, cm_volume_t **outVolpp)
 {
     cm_volume_t *volp;
@@ -613,12 +673,12 @@ long cm_GetVolumeByID(cm_cell_t *cellp, afs_uint32 volumeID, cm_user_t *userp,
     osi_assertx(volp == volp2, "unexpected cm_vol_t");
 #endif
 
-    lock_ReleaseRead(&cm_volumeLock);
-
     /* hold the volume if we found it */
     if (volp) 
         cm_GetVolume(volp);
         
+    lock_ReleaseRead(&cm_volumeLock);
+
     /* return it held */
     if (volp) {
         lock_ObtainMutex(&volp->mx);
@@ -646,13 +706,13 @@ long cm_GetVolumeByID(cm_cell_t *cellp, afs_uint32 volumeID, cm_user_t *userp,
         
     /* otherwise, we didn't find it so consult the VLDB */
     sprintf(volNameString, "%u", volumeID);
-    code = cm_GetVolumeByName(cellp, volNameString, userp, reqp,
+    code = cm_FindVolumeByName(cellp, volNameString, userp, reqp,
 			      flags, outVolpp);
     return code;
 }
 
 
-long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
+long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
 			struct cm_user *userp, struct cm_req *reqp,
 			afs_uint32 flags, cm_volume_t **outVolpp)
 {
@@ -752,11 +812,9 @@ long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
                 cm_VolumeStatusNotification(volp, volp->bk.ID, volp->bk.state, vl_unknown);
 
             volp->rw.ID = volp->ro.ID = volp->bk.ID = 0;
-	    volp->dotdotFid.cell = 0;
-	    volp->dotdotFid.volume = 0;
-	    volp->dotdotFid.unique = 0;
-	    volp->dotdotFid.vnode = 0;
-            volp->dotdotFid.hash = 0;
+            cm_SetFid(&volp->rw.dotdotFid, 0, 0, 0, 0);
+            cm_SetFid(&volp->ro.dotdotFid, 0, 0, 0, 0);
+            cm_SetFid(&volp->bk.dotdotFid, 0, 0, 0, 0);
 	} else {
 	    volp = &cm_data.volumeBaseAddress[cm_data.currentVolumes++];
 	    memset(volp, 0, sizeof(cm_volume_t));
@@ -781,13 +839,14 @@ long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
         lock_ReleaseWrite(&cm_volumeLock);
     }
     else {
-        lock_ReleaseRead(&cm_volumeLock);
-        if (volp) {
+        if (volp)
             cm_GetVolume(volp);
-            lock_ObtainMutex(&volp->mx);
-        } else {
+        lock_ReleaseRead(&cm_volumeLock);
+        
+        if (!volp)
             return CM_ERROR_NOSUCHVOLUME;
-        }
+
+        lock_ObtainMutex(&volp->mx);
     }
 
     /* if we get here we are holding the mutex */
@@ -803,15 +862,15 @@ long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
         code = CM_ERROR_NOSUCHVOLUME;
 
     if (code == 0) {
-		*outVolpp = volp;
+        *outVolpp = volp;
 		
-		if (!(flags & CM_GETVOL_FLAG_NO_LRU_UPDATE)) {
-	        lock_ObtainWrite(&cm_volumeLock);
-			cm_AdjustVolumeLRU(volp);
-			lock_ReleaseWrite(&cm_volumeLock);
-		}
+        if (!(flags & CM_GETVOL_FLAG_NO_LRU_UPDATE)) {
+            lock_ObtainWrite(&cm_volumeLock);
+            cm_AdjustVolumeLRU(volp);
+            lock_ReleaseWrite(&cm_volumeLock);
+        }
     } else
-		cm_PutVolume(volp);
+        cm_PutVolume(volp);
 
     return code;
 }	
@@ -869,13 +928,12 @@ void cm_ForceUpdateVolume(cm_fid_t *fidp, cm_user_t *userp, cm_req_t *reqp)
 #ifdef SEARCH_ALL_VOLUMES
     osi_assertx(volp == volp2, "unexpected cm_vol_t");
 #endif
-
-    lock_ReleaseRead(&cm_volumeLock);
-
     /* hold the volume if we found it */
     if (volp) 
 	cm_GetVolume(volp);
 
+    lock_ReleaseRead(&cm_volumeLock);
+
     /* update it */
     cm_data.mountRootGen = time(NULL);
     lock_ObtainMutex(&volp->mx);
@@ -927,9 +985,8 @@ cm_serverRef_t **cm_GetVolServers(cm_volume_t *volp, afs_uint32 volume)
 
 void cm_PutVolume(cm_volume_t *volp)
 {
-    lock_ObtainWrite(&cm_volumeLock);
-    osi_assertx(volp->refCount-- > 0, "cm_volume_t refCount 0");
-    lock_ReleaseWrite(&cm_volumeLock);
+    afs_int32 refCount = InterlockedDecrement(&volp->refCount);
+    osi_assertx(refCount >= 0, "cm_volume_t refCount underflow has occurred");
 }
 
 /* return the read-only volume, if there is one, or the read-write volume if
@@ -953,23 +1010,25 @@ void cm_RefreshVolumes(void)
 {
     cm_volume_t *volp;
     cm_scache_t *scp;
+    afs_int32 refCount;
 
     cm_data.mountRootGen = time(NULL);
 
     /* force a re-loading of volume data from the vldb */
-    lock_ObtainWrite(&cm_volumeLock);
+    lock_ObtainRead(&cm_volumeLock);
     for (volp = cm_data.allVolumesp; volp; volp=volp->allNextp) {
-	volp->refCount++;
-	lock_ReleaseWrite(&cm_volumeLock);
+	InterlockedIncrement(&volp->refCount);
+	lock_ReleaseRead(&cm_volumeLock);
 
 	lock_ObtainMutex(&volp->mx);
 	volp->flags |= CM_VOLUMEFLAG_RESET;
 	lock_ReleaseMutex(&volp->mx);
 	
-        lock_ObtainWrite(&cm_volumeLock);
-	osi_assertx(volp->refCount-- > 0, "cm_volume_t refCount 0");
+        lock_ObtainRead(&cm_volumeLock);
+        refCount = InterlockedDecrement(&volp->refCount);
+	osi_assertx(refCount >= 0, "cm_volume_t refCount underflow");
     }
-    lock_ReleaseWrite(&cm_volumeLock);
+    lock_ReleaseRead(&cm_volumeLock);
 
     /* force mount points to be re-evaluated so that 
      * if the volume location has changed we will pick 
@@ -983,9 +1042,9 @@ void cm_RefreshVolumes(void)
              && !(scp->fid.cell == AFS_FAKE_ROOT_CELL_ID && scp->fid.volume == AFS_FAKE_ROOT_VOL_ID)
 #endif
              ) {
-            lock_ObtainMutex(&scp->mx);
+            lock_ObtainWrite(&scp->rw);
             scp->mountPointStringp[0] = '\0';
-            lock_ReleaseMutex(&scp->mx);
+            lock_ReleaseWrite(&scp->rw);
         }
     }
 
@@ -1142,18 +1201,20 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
 void cm_CheckOfflineVolumes(void)
 {
     cm_volume_t *volp;
+    afs_int32 refCount;
 
-    lock_ObtainWrite(&cm_volumeLock);
+    lock_ObtainRead(&cm_volumeLock);
     for (volp = cm_data.allVolumesp; volp; volp=volp->allNextp) {
-	volp->refCount++;
-	lock_ReleaseWrite(&cm_volumeLock);
-
-        cm_CheckOfflineVolume(volp, 0);
-
-	lock_ObtainWrite(&cm_volumeLock);
-	osi_assertx(volp->refCount-- > 0, "cm_volume_t refCount 0");
+        if (volp->flags & CM_VOLUMEFLAG_IN_HASH) {
+            InterlockedIncrement(&volp->refCount);
+            lock_ReleaseRead(&cm_volumeLock);
+            cm_CheckOfflineVolume(volp, 0);
+            lock_ObtainRead(&cm_volumeLock);
+            refCount = InterlockedDecrement(&volp->refCount);
+            osi_assertx(refCount >= 0, "cm_volume_t refCount underflow");
+        }
     }
-    lock_ReleaseWrite(&cm_volumeLock);
+    lock_ReleaseRead(&cm_volumeLock);
 }
 
 
@@ -1243,14 +1304,15 @@ void cm_ChangeRankVolume(cm_server_t *tsp)
 {	
     int 		code;
     cm_volume_t*	volp;
+    afs_int32 refCount;
 
     /* find volumes which might have RO copy on server*/
-    lock_ObtainWrite(&cm_volumeLock);
+    lock_ObtainRead(&cm_volumeLock);
     for(volp = cm_data.allVolumesp; volp; volp=volp->allNextp)
     {
 	code = 1 ;	/* assume that list is unchanged */
-	volp->refCount++;
-	lock_ReleaseWrite(&cm_volumeLock);
+	InterlockedIncrement(&volp->refCount);
+	lock_ReleaseRead(&cm_volumeLock);
 	lock_ObtainMutex(&volp->mx);
 
 	if ((tsp->cellp==volp->cellp) && (volp->ro.serversp))
@@ -1261,10 +1323,11 @@ void cm_ChangeRankVolume(cm_server_t *tsp)
 	    cm_RandomizeServer(&volp->ro.serversp);
 
 	lock_ReleaseMutex(&volp->mx);
-	lock_ObtainWrite(&cm_volumeLock);
-	osi_assertx(volp->refCount-- > 0, "cm_volume_t refCount 0");
+	lock_ObtainRead(&cm_volumeLock);
+        refCount = InterlockedDecrement(&volp->refCount);
+	osi_assertx(refCount >= 0, "cm_volume_t refCount underflow");
     }
-    lock_ReleaseWrite(&cm_volumeLock);
+    lock_ReleaseRead(&cm_volumeLock);
 }	
 
 /* dump all volumes that have reference count > 0 to a file. 
@@ -1287,19 +1350,9 @@ int cm_DumpVolumes(FILE *outputFile, char *cookie, int lock)
   
     for (volp = cm_data.allVolumesp; volp; volp=volp->allNextp)
     {
-        cm_scache_t *scp;
-        int scprefs = 0;
-
-        for (scp = cm_data.allSCachesp; scp; scp = scp->allNextp) 
-        {
-            if (scp->volp == volp)
-                scprefs++;
-        }   
-
-        sprintf(output, "%s - volp=0x%p cell=%s name=%s rwID=%u roID=%u bkID=%u flags=0x%x fid (cell=%d, volume=%d, vnode=%d, unique=%d) refCount=%u scpRefs=%u\r\n", 
+        sprintf(output, "%s - volp=0x%p cell=%s name=%s rwID=%u roID=%u bkID=%u flags=0x%x refCount=%u\r\n", 
                  cookie, volp, volp->cellp->name, volp->namep, volp->rw.ID, volp->ro.ID, volp->bk.ID, volp->flags, 
-                 volp->dotdotFid.cell, volp->dotdotFid.volume, volp->dotdotFid.vnode, volp->dotdotFid.unique,
-                 volp->refCount, scprefs);
+                 volp->refCount);
         WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
     }
     sprintf(output, "%s - Done dumping volumes.\r\n", cookie);
@@ -1494,6 +1547,22 @@ void cm_AdjustVolumeLRU(cm_volume_t *volp)
         cm_data.volumeLRULastp = volp;
 }
 
+/* must be called with cm_volumeLock write-locked! */
+void cm_MoveVolumeToLRULast(cm_volume_t *volp)
+{
+    if (volp == cm_data.volumeLRULastp)
+        return;
+
+    if (volp == cm_data.volumeLRUFirstp)
+        cm_data.volumeLRUFirstp = (cm_volume_t *) osi_QNext(&volp->q);
+    if (volp->flags & CM_VOLUMEFLAG_IN_LRU_QUEUE)
+        osi_QRemoveHT((osi_queue_t **) &cm_data.volumeLRUFirstp, (osi_queue_t **) &cm_data.volumeLRULastp, &volp->q);
+    osi_QAddT((osi_queue_t **) &cm_data.volumeLRUFirstp, (osi_queue_t **) &cm_data.volumeLRULastp, &volp->q);
+    volp->flags |= CM_VOLUMEFLAG_IN_LRU_QUEUE;
+    if (!cm_data.volumeLRULastp) 
+        cm_data.volumeLRULastp = volp;
+}
+
 /* must be called with cm_volumeLock write-locked! */
 void cm_RemoveVolumeFromLRU(cm_volume_t *volp)
 {
@@ -1577,9 +1646,9 @@ cm_VolumeRenewROCallbacks(void)
 
             lock_ReleaseRead(&cm_volumeLock);
             if (cm_GetSCache(&fid, &scp, cm_rootUserp, &req) == 0) {
-                lock_ObtainMutex(&scp->mx);
+                lock_ObtainWrite(&scp->rw);
                 cm_GetCallback(scp, cm_rootUserp, &req, 1);
-                lock_ReleaseMutex(&scp->mx);
+                lock_ReleaseWrite(&scp->rw);
                 cm_ReleaseSCache(scp);
             }
             lock_ObtainRead(&cm_volumeLock);
