@@ -15,8 +15,9 @@
 #define CM_VOLUME_MAGIC    ('V' | 'O' <<8 | 'L'<<16 | 'M'<<24)
 
 typedef struct cm_vol_state {
-    afs_uint32      ID;                 /* by mx */
     struct cm_volume *nextp;            /* volumeIDHashTable; by cm_volumeLock */
+    afs_uint32      ID;                 /* by mx */
+    struct cm_fid dotdotFid;	        /* parent of volume root */
     cm_serverRef_t *serversp;           /* by mx */
     enum volstatus  state;              /* by mx */
     afs_uint32      flags;              /* by mx */
@@ -33,10 +34,9 @@ typedef struct cm_volume {
     struct cm_vol_state rw;	        /* by cm_volumeLock */
     struct cm_vol_state ro;		/* by cm_volumeLock */
     struct cm_vol_state bk;		/* by cm_volumeLock */
-    struct cm_fid dotdotFid;	        /* parent of volume root */
     osi_mutex_t mx;
     afs_uint32 flags;			/* by mx */
-    afs_uint32 refCount;		/* by cm_volumeLock */
+    afs_int32 refCount;		        /* by Interlocked operations */
     time_t cbExpiresRO;                 /* latest RO expiration time; by cm_scacheLock */
 } cm_volume_t;
 
@@ -52,11 +52,11 @@ typedef struct cm_volumeRef {
 
 extern void cm_InitVolume(int newFile, long maxVols);
 
-extern long cm_GetVolumeByName(struct cm_cell *cellp, char *volNamep, 
+extern long cm_FindVolumeByName(struct cm_cell *cellp, char *volNamep, 
                                struct cm_user *userp, struct cm_req *reqp, 
                                afs_uint32 flags, cm_volume_t **outVolpp);
 
-extern long cm_GetVolumeByID(struct cm_cell *cellp, afs_uint32 volumeID,
+extern long cm_FindVolumeByID(struct cm_cell *cellp, afs_uint32 volumeID,
                              cm_user_t *userp, cm_req_t *reqp, 
                              afs_uint32 flags, cm_volume_t **outVolpp);
 
@@ -77,6 +77,8 @@ extern afs_uint32 SDBMHash(const char *);
 
 extern void cm_GetVolume(cm_volume_t *volp);
 
+extern cm_volume_t *cm_GetVolumeByFID(cm_fid_t *);
+
 extern void cm_PutVolume(cm_volume_t *volp);
 
 extern long cm_GetROVolumeID(cm_volume_t *volp);
@@ -108,6 +110,8 @@ extern void cm_AddVolumeToIDHashTable(cm_volume_t * volp, afs_uint32 volType);
 
 extern void cm_AdjustVolumeLRU(cm_volume_t *volp);
 
+extern void cm_MoveVolumeToLRULast(cm_volume_t *volp);
+
 extern void cm_RemoveVolumeFromLRU(cm_volume_t *volp);
 
 extern void cm_CheckOfflineVolumes(void);
@@ -121,4 +125,6 @@ extern void cm_VolumeStatusNotification(cm_volume_t * volp, afs_uint32 volID, en
 extern enum volstatus cm_GetVolumeStatus(cm_volume_t *volp, afs_uint32 volID);
 
 extern void cm_VolumeRenewROCallbacks(void);
+
+extern osi_rwlock_t cm_volumeLock;
 #endif /*  __CM_VOLUME_H_ENV__ */
