@@ -41,11 +41,6 @@ struct sbstruct {
         int sb_default;
 };
 
-#define CM_IOCTLCACHEPARMS		16
-typedef struct cm_cacheParms {
-	long parms[CM_IOCTLCACHEPARMS];
-} cm_cacheParms_t;
-
 /* set cell flags */
 #define CM_SETCELLFLAG_SUID		2
 
@@ -87,9 +82,19 @@ typedef struct cm_cacheParms {
 #define VIOC_LISTSYMLINK		0x24
 #define VIOC_DELSYMLINK			0x25
 #define VIOC_MAKESUBMOUNT		0x26
-#define VIOC_SHUTDOWN           0x27
-
 #define VIOC_GETRXKCRYPT		0x27
 #define VIOC_SETRXKCRYPT		0x28
-#define VIOC_TRACEMEMDUMP       0x29
+#define VIOC_TRACEMEMDUMP               0x29
+#define VIOC_SHUTDOWN                   0x2a
+#define VIOC_FLUSHALL                   0x2b
+#define VIOCGETFID                      0x2c
+#define VIOCGETOWNER                    0x2d
+#define VIOC_RXSTAT_PROC                0x2e
+#define VIOC_RXSTAT_PEER                0x2f
+#define VIOC_UUIDCTL                    0x30
+#define VIOC_PATH_AVAILABILITY          0x31
+#define VIOC_GETFILETYPE                0x32
+
+#define VIOC_VOLSTAT_TEST               0x3F
+/* Not to exceed SMB_IOCTL_MAXPROCS from smb_ioctl.h */
 #endif /*  __SMB_IOCONS_H_ENV_ */
