@@ -20,7 +20,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_groups.c,v 1.28.4.11 2008/01/30 17:30:41 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_groups.c,v 1.28.4.12 2008/03/10 16:47:18 shadow Exp $");
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
@@ -223,11 +223,8 @@ __setpag(cred_t **cr, afs_uint32 pagvalue, afs_uint32 *newpag,
 }
 
 #ifdef LINUX_KEYRING_SUPPORT
-#ifdef EXPORTS_KEY_TYPE_KEYRING
+extern struct key_type key_type_keyring __attribute__((weak));
 static struct key_type *__key_type_keyring = &key_type_keyring;
-#else
-static struct key_type *__key_type_keyring;
-#endif
 
 static int
 install_session_keyring(struct task_struct *task, struct key *keyring)
@@ -627,32 +624,33 @@ extern rwlock_t tasklist_lock __attribute__((weak));
 
 void osi_keyring_init(void)
 {
-#ifndef EXPORTS_KEY_TYPE_KEYRING
     struct task_struct *p;
+    
+    if (__key_type_keyring == NULL) {
 #ifdef EXPORTED_TASKLIST_LOCK
-    if (&tasklist_lock)
-      read_lock(&tasklist_lock);
+	if (&tasklist_lock)
+	    read_lock(&tasklist_lock);
 #endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
-#ifdef EXPORTED_TASKLIST_LOCK
-    else
+# ifdef EXPORTED_TASKLIST_LOCK
+ 	else
+# endif
+	    rcu_read_lock();
 #endif
-      rcu_read_lock();
-#endif
-    p = find_task_by_pid(1);
-    if (p && p->user->session_keyring)
-	__key_type_keyring = p->user->session_keyring->type;
+	p = find_task_by_pid(1);
+	if (p && p->user->session_keyring)
+	    __key_type_keyring = p->user->session_keyring->type;
 #ifdef EXPORTED_TASKLIST_LOCK
-    if (&tasklist_lock)
-       read_unlock(&tasklist_lock);
+	if (&tasklist_lock)
+	    read_unlock(&tasklist_lock);
 #endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
-#ifdef EXPORTED_TASKLIST_LOCK
-    else
-#endif
-      rcu_read_unlock();
-#endif
+# ifdef EXPORTED_TASKLIST_LOCK
+	else
+# endif
+	    rcu_read_unlock();
 #endif
+    }
 
     register_key_type(&key_type_afs_pag);
 }
