@@ -21,7 +21,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_dirops.c,v 1.14.2.3 2004/11/09 17:15:04 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_dirops.c,v 1.21.4.1 2007/12/08 17:59:07 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -36,24 +36,9 @@ extern afs_rwlock_t afs_xcbhash;
 /* don't set CDirty in here because RPC is called synchronously */
 
 int
-#ifdef	AFS_OSF_ENV
-afs_mkdir(ndp, attrs)
-     struct nameidata *ndp;
-     struct vattr *attrs;
+afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs, 
+     register struct vcache **avcp, struct AFS_UCRED *acred)
 {
-    register struct vcache *adp = VTOAFS(ndp->ni_dvp);
-    char *aname = ndp->ni_dent.d_name;
-    register struct vcache **avcp = (struct vcache **)&(ndp->ni_vp);
-    struct ucred *acred = ndp->ni_cred;
-#else /* AFS_OSF_ENV */
-afs_mkdir(OSI_VC_ARG(adp), aname, attrs, avcp, acred)
-     OSI_VC_DECL(adp);
-     register struct vcache **avcp;
-     char *aname;
-     struct vattr *attrs;
-     struct AFS_UCRED *acred;
-{
-#endif
     struct vrequest treq;
     register afs_int32 code;
     register struct conn *tc;
@@ -147,7 +132,9 @@ afs_mkdir(OSI_VC_ARG(adp), aname, attrs, avcp, acred)
 	ObtainWriteLock(&tdc->lock, 632);
     if (afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
 	/* we can do it locally */
+	ObtainWriteLock(&afs_xdcache, 294);
 	code = afs_dir_Create(tdc, aname, &newFid.Fid);
+	ReleaseWriteLock(&afs_xdcache);
 	if (code) {
 	    ZapDCE(tdc);	/* surprise error -- use invalid value */
 	    DZap(tdc);
@@ -172,34 +159,19 @@ afs_mkdir(OSI_VC_ARG(adp), aname, attrs, avcp, acred)
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 26);
   done2:
-#ifdef	AFS_OSF_ENV
-    AFS_RELE(ndp->ni_dvp);
-#endif /* AFS_OSF_ENV */
     return code;
 }
 
 
 int
-#ifdef	AFS_OSF_ENV
-afs_rmdir(ndp)
-     struct nameidata *ndp;
-{
-    register struct vcache *adp = VTOAFS(ndp->ni_dvp);
-    char *aname = ndp->ni_dent.d_name;
-    struct ucred *acred = ndp->ni_cred;
-#else /* AFS_OSF_ENV */
 /* don't set CDirty in here because RPC is called synchronously */
 #if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
-afs_rmdir(OSI_VC_ARG(adp), aname, cdirp, acred)
-     struct vnode *cdirp;
+afs_rmdir(OSI_VC_DECL(adp), char *aname, struct vnode *cdirp, 
+	  struct AFS_UCRED *acred)
 #else
-afs_rmdir(adp, aname, acred)
+afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
 #endif
-     OSI_VC_DECL(adp);
-     char *aname;
-     struct AFS_UCRED *acred;
 {
-#endif
     struct vrequest treq;
     register struct dcache *tdc;
     register struct vcache *tvc = NULL;
@@ -316,10 +288,9 @@ afs_rmdir(adp, aname, acred)
     }
 
 
-    if (tvc) {
+    if (tvc)
 	osi_dnlc_purgedp(tvc);	/* get rid of any entries for this directory */
-	afs_symhint_inval(tvc);
-    } else
+    else
 	osi_dnlc_remove(adp, aname, 0);
 
     if (tvc) {
@@ -336,9 +307,5 @@ afs_rmdir(adp, aname, acred)
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 27);
   done2:
-#ifdef	AFS_OSF_ENV
-    afs_PutVCache(adp);
-    afs_PutVCache(ndp->ni_vp);
-#endif /* AFS_OSF_ENV */
     return code;
 }
