@@ -16,7 +16,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_util.c,v 1.28.2.2 2006/06/15 15:45:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_util.c,v 1.28.2.7 2008/02/11 04:00:47 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -136,6 +136,19 @@ afs_strchr(char *s, int c)
     return NULL;
 }
 #endif
+#ifndef afs_strrchr
+char *
+afs_strrchr(char *s, int c)
+{
+    char *p = NULL;
+    
+    do {
+	if (*s == c)
+	    p = (char*) s;
+    } while (*s++);
+    return p;
+}
+#endif
 
 char *
 afs_strdup(char *s)
@@ -561,7 +574,7 @@ calc (struct afs_md5 *m, afs_uint32 *data)
  * From `Performance analysis of MD5' by Joseph D. Touch <touch@isi.edu>
  */
 
-#if defined(AFSBIG_ENDIAN)
+#if defined(WORDS_BIGENDIAN)
 static inline afs_uint32
 swap_u_int32_t (afs_uint32 t)
 {
@@ -599,7 +612,7 @@ AFS_MD5_Update (struct afs_md5 *m, const void *v, size_t len)
 	p += l;
 	len -= l;
 	if(offset == 64){
-#if defined(AFSBIG_ENDIAN)
+#if defined(WORDS_BIGENDIAN)
 	    int i;
 	    afs_uint32 temp[16];
 	    struct x32 *us = (struct x32*)m->save;
@@ -649,7 +662,7 @@ AFS_MD5_Final (void *res, struct afs_md5 *m)
 
 afs_int32 afs_calc_inum (afs_int32 volume, afs_int32 vnode)
 { 
-    afs_int32 ino;
+    afs_int32 ino, vno = vnode;
     char digest[16];
     struct afs_md5 ct;
     
@@ -659,10 +672,11 @@ afs_int32 afs_calc_inum (afs_int32 volume, afs_int32 vnode)
 	AFS_MD5_Update(&ct, &vnode, 4);
 	AFS_MD5_Final(digest, &ct);
 	memcpy(&ino, digest, sizeof(ino_t));
+	ino ^= (ino ^ vno) & 1;
     } else {
 	ino = (volume << 16) + vnode;
-	ino &= 0x7fffffff;      /* Assumes 32 bit ino_t ..... */
     }
+    ino &= 0x7fffffff;      /* Assumes 32 bit ino_t ..... */
     return ino;
 }
 
