@@ -1,4 +1,4 @@
-/* $Id: pt_util.c,v 1.11 2005/07/15 19:16:27 shadow Exp $ */
+/* $Id: pt_util.c,v 1.11.4.3 2008/03/09 01:58:49 jaltman Exp $ */
 
 /*
  *
@@ -23,7 +23,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/pt_util.c,v 1.11 2005/07/15 19:16:27 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/pt_util.c,v 1.11.4.3 2008/03/09 01:58:49 jaltman Exp $");
 
 #include <afs/cmd.h>		/*Command line parsing */
 #include <errno.h>
@@ -33,6 +33,7 @@ RCSID
 #include <ubik.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
+#include <afs/com_err.h>
 #include "ptint.h"
 #include "ptserver.h"
 #include "pterror.h"
@@ -53,7 +54,7 @@ void fix_pre();
 char *checkin();
 char *check_core();
 char *id_to_name();
-int CommandProc(struct cmd_syndesc *);
+int CommandProc(struct cmd_syndesc *, void *);
 
 struct hash_entry {
     char h_name[PR_MAXNAMELEN];
@@ -105,7 +106,7 @@ main(int argc, char **argv)
     register struct cmd_syndesc *cs;	/*Command line syntax descriptor */
     register afs_int32 code;	/*Return code */
 
-    cs = cmd_CreateSyntax((char *)0, CommandProc, 0,
+    cs = cmd_CreateSyntax(NULL, CommandProc, NULL,
 			  "access protection database");
     cmd_AddParm(cs, "-w", CMD_FLAG, CMD_OPTIONAL,
 		"update prdb with contents of data file");
@@ -128,7 +129,7 @@ main(int argc, char **argv)
 }
 
 int
-CommandProc(register struct cmd_syndesc *a_as)
+CommandProc(register struct cmd_syndesc *a_as, void *arock)
 {
     register int i;
     register long code;
@@ -242,7 +243,7 @@ CommandProc(register struct cmd_syndesc *a_as)
 			    if (code)
 				fprintf(stderr,
 					"Error setting group count on %s: %s\n",
-					name, error_message(code));
+					name, afs_error_message(code));
 			}
 			code = CreateEntry(0, u->name, &uid, 1 /*idflag */ ,
 					   1 /*gflag */ ,
@@ -251,7 +252,7 @@ CommandProc(register struct cmd_syndesc *a_as)
 		    }
 		    if (code)
 			fprintf(stderr, "Error while creating %s: %s\n",
-				u->name, error_message(code));
+				u->name, afs_error_message(code));
 		    continue;
 		}
 		/* Add user to group */
@@ -271,7 +272,7 @@ CommandProc(register struct cmd_syndesc *a_as)
 
 		if (code)
 		    fprintf(stderr, "Error while adding %s to %s: %s\n", mem,
-			    name, error_message(code));
+			    name, afs_error_message(code));
 	    } else {
 		sscanf(buffer, "%s %d/%d %d %d %d", name, &flags, &quota, &id,
 		       &oid, &cid);
@@ -289,7 +290,7 @@ CommandProc(register struct cmd_syndesc *a_as)
 		    usr_head = u;
 		} else if (code) {
 		    fprintf(stderr, "Error while creating %s: %s\n", name,
-			    error_message(code));
+			    afs_error_message(code));
 		} else if ((flags & PRACCESS)
 			   || (flags & (PRGRP | PRQUOTA)) ==
 			   (PRGRP | PRQUOTA)) {
@@ -303,14 +304,14 @@ CommandProc(register struct cmd_syndesc *a_as)
 		    if (code)
 			fprintf(stderr,
 				"Error while setting flags on %s: %s\n", name,
-				error_message(code));
+				afs_error_message(code));
 		}
 	    }
 	}
 	for (u = usr_head; u; u = u->next)
 	    if (u->uid)
 		fprintf(stderr, "Error while creating %s: %s\n", u->name,
-			error_message(PRBADNAM));
+			afs_error_message(PRBADNAM));
     } else {
 	for (i = 0; i < HASHSIZE; i++) {
 	    upos = nflag ? ntohl(prh.nameHash[i]) : ntohl(prh.idHash[i]);
