@@ -22,6 +22,7 @@
 #ifndef GLOBALSINIT
 #define GLOBALSINIT(x)
 #if defined(AFS_NT40_ENV)
+#define RX_STATS_INTERLOCKED 1
 #if defined(AFS_PTHREAD_ENV)
 #define EXT __declspec(dllimport) extern
 #else
@@ -202,6 +203,8 @@ EXT struct rx_queue rx_freePacketQueue;
         if ((p)->flags & RX_PKTFLAG_FREE) \
             osi_Panic("rx packet already free\n"); \
         (p)->flags |= RX_PKTFLAG_FREE; \
+        (p)->length = 0; \
+        (p)->niovecs = 0; \
     } while(0)
 #define RX_FPQ_MARK_USED(p) \
     do { \
@@ -343,7 +346,7 @@ EXT void rxi_FlushLocalPacketsTSFPQ(void); /* flush all thread-local packets to
         (rx_ts_info_p)->_FPQ.checkout_xfer++; \
     } while(0)
 /* checkout multiple packets from the thread-specific free packet queue */
-#define RX_TS_FPQ_CHECKOUT2(rx_ts_info_p,num_transfer,q) \
+#define RX_TS_FPQ_QCHECKOUT(rx_ts_info_p,num_transfer,q) \
     do { \
         register int i; \
         register struct rx_packet *p; \
@@ -370,7 +373,7 @@ EXT void rxi_FlushLocalPacketsTSFPQ(void); /* flush all thread-local packets to
 /* num_transfer must equal length of (q); it is not a means of checking 
  * in part of (q).  passing num_transfer just saves us instructions 
  * since caller already knows length of (q) for other reasons */
-#define RX_TS_FPQ_CHECKIN2(rx_ts_info_p,num_transfer,q) \
+#define RX_TS_FPQ_QCHECKIN(rx_ts_info_p,num_transfer,q) \
     do { \
         register struct rx_packet *p, *np; \
         for (queue_Scan((q), p, np, rx_packet)) { \
