@@ -15,7 +15,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_rdwr.c,v 1.29.2.2 2007/10/30 15:16:45 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_rdwr.c,v 1.29.2.3 2008/03/17 15:38:28 shadow Exp $");
 
 #ifdef KERNEL
 #ifndef UKERNEL
@@ -192,8 +192,9 @@ rxi_ReadProc(register struct rx_call *call, register char *buf,
 					       RX_CALL_REFCOUNT_DELAY);
 				rxi_SendAck(call, 0, 0, RX_ACK_DELAY, 0);
 			    } else {
-				struct clock when;
-				clock_GetTime(&when);
+				struct clock when, now;
+				clock_GetTime(&now);
+				when = now;
 				/* Delay to consolidate ack packets */
 				clock_Add(&when, &rx_hardAckDelay);
 				if (!call->delayedAckEvent
@@ -204,7 +205,7 @@ rxi_ReadProc(register struct rx_call *call, register char *buf,
 						   RX_CALL_REFCOUNT_DELAY);
 				    CALL_HOLD(call, RX_CALL_REFCOUNT_DELAY);
 				    call->delayedAckEvent =
-					rxevent_Post(&when,
+				      rxevent_PostNow(&when, &now,
 						     rxi_SendDelayedAck, call,
 						     0);
 				}
@@ -521,8 +522,9 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 	    rxi_SendAck(call, 0, serial, RX_ACK_DELAY, 0);
 	    didHardAck = 1;
 	} else {
-	    struct clock when;
-	    clock_GetTime(&when);
+	    struct clock when, now;
+	    clock_GetTime(&now);
+	    when = now;
 	    /* Delay to consolidate ack packets */
 	    clock_Add(&when, &rx_hardAckDelay);
 	    if (!call->delayedAckEvent
@@ -531,7 +533,7 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 			       RX_CALL_REFCOUNT_DELAY);
 		CALL_HOLD(call, RX_CALL_REFCOUNT_DELAY);
 		call->delayedAckEvent =
-		    rxevent_Post(&when, rxi_SendDelayedAck, call, 0);
+		    rxevent_PostNow(&when, &now, rxi_SendDelayedAck, call, 0);
 	    }
 	}
     }
