@@ -13,11 +13,12 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_user.c,v 1.18.2.3 2005/03/11 07:03:33 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_user.c,v 1.24.4.2 2007/10/30 15:16:45 shadow Exp $");
 
 # include <sys/types.h>
 # include <errno.h>
 # include <signal.h>
+# include <string.h>
 #ifdef AFS_NT40_ENV
 # include <WINNT/syscfg.h>
 #else
@@ -36,13 +37,6 @@ RCSID
 #endif
 #include <afs/afs_args.h>
 #include <afs/afsutil.h>
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #ifndef	IPPORT_USERRESERVED
 /* If in.h doesn't define this, define it anyway.  Unfortunately, defining
@@ -213,7 +207,8 @@ rxi_GetUDPSocket(u_short port)
 }
 
 void
-osi_Panic(char *msg, int a1, int a2, int a3)
+osi_Panic(msg, a1, a2, a3) 
+     char *msg; 
 {
     (osi_Msg "Fatal Rx error: ");
     (osi_Msg msg, a1, a2, a3);
@@ -229,34 +224,34 @@ osi_Panic(char *msg, int a1, int a2, int a3)
 void
 osi_AssertFailU(const char *expr, const char *file, int line)
 {
-    osi_Panic("assertion failed: %s, file: %s, line: %d\n", (int)expr,
-	      (int)file, line);
+    osi_Panic("assertion failed: %s, file: %s, line: %d\n", expr,
+	      file, line);
 }
 
-#ifdef	AFS_AIX32_ENV
+#if defined(AFS_AIX32_ENV) && !defined(KERNEL)
 #ifndef osi_Alloc
 static const char memZero;
-char *
+void *
 osi_Alloc(afs_int32 x)
 {
     /* 
-     * 0-length allocs may return NULL ptr from osi_kalloc, so we special-case
+     * 0-length allocs may return NULL ptr from malloc, so we special-case
      * things so that NULL returned iff an error occurred 
      */
     if (x == 0)
-	return &memZero;
-    return ((char *)malloc(x));
+	return (void *)&memZero;
+    return(malloc(x));
 }
 
 void
-osi_Free(char *x, afs_int32 size)
+osi_Free(void *x, afs_int32 size)
 {
     if (x == &memZero)
 	return;
-    free((char *)x);
+    free(x);
 }
 #endif
-#endif /* AFS_AIX32_ENV */
+#endif /* defined(AFS_AIX32_ENV) && !defined(KERNEL) */
 
 #define	ADDRSPERSITE	16
 
@@ -298,10 +293,14 @@ rx_getAllAddr(afs_int32 * buffer, int maxSize)
     /* The IP address list can change so we must query for it */
     rx_GetIFInfo();
 
+#ifndef AFS_NT40_ENV
     /* we don't want to use the loopback adapter which is first */
-    /* this is a bad bad hack */
+    /* this is a bad bad hack.
+     * and doesn't hold true on Windows.
+     */
     if ( rxi_numNetAddrs > 1 )
         offset = 1;
+#endif /* AFS_NT40_ENV */
 
     for (count = 0; offset < rxi_numNetAddrs && maxSize > 0;
 	 count++, offset++, maxSize--)
