@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vlserver/vlclient.c,v 1.12 2003/12/07 22:49:42 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/vlserver/vlclient.c,v 1.16.2.3 2007/10/31 04:09:43 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -38,21 +38,13 @@ RCSID
 #include <netinet/in.h>
 #endif
 #include <stdio.h>
-
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/afsutil.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rx_globals.h>
 #include <rx/rxkad.h>
-#include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <afs/keys.h>
 #include <afs/cmd.h>
@@ -162,102 +154,17 @@ GetVolume(vol, entry)
 
 /* Almost identical's to pr_Initialize in vlserver/pruser.c */
 afs_int32
-vl_Initialize(auth, confDir, server, cellp)
-     int auth, server;
-     char *confDir, *cellp;
+vl_Initialize(int auth, char *confDir, int server, char *cellp)
 {
-    afs_int32 code;
-    struct afsconf_dir *tdir;
-    struct ktc_principal sname;
-    struct ktc_token ttoken;
-    afs_int32 scIndex = 0;
-    struct rx_securityClass *sc;
-    struct afsconf_cell info;
-    afs_int32 i;
-
-    code = rx_Init(0);
-    if (code) {
-	fprintf(stderr, "vl_Initialize:  Could not initialize rx.\n");
-	return code;
-    }
-
-    rx_SetRxDeadTime(50);
-    if (!server) {
-	tdir = afsconf_Open(confDir);
-	if (!tdir) {
-	    fprintf(stderr, "Could not open configuration directory (%s).\n",
-		    confDir);
-	    return -1;
-	}
-    }
-    if (auth) {			/* we don't need tickets for null */
-	if (!server) {
-	    code = afsconf_GetLocalCell(tdir, sname.cell, sizeof(sname.cell));
-	    if (code) {
-		fprintf(stderr,
-			"vl_Initialize: Could not get local cell name.\n");
-		return code;
-	    }
-	} else
-	    strncpy(sname.cell, cellp, sizeof(sname.cell));
-	sname.instance[0] = 0;
-	strcpy(sname.name, "afs");
-	code = ktc_GetToken(&sname, &ttoken, sizeof(ttoken), NULL);
-	if (code) {
-	    fprintf(stderr,
-		    "vl_Initialize: Could not get afs tokens, running unauthenticated.\n");
-	    scIndex = 0;
-	} else if (ttoken.kvno <= 255)
-	    scIndex = 2;
-	else {			/* bcrypt */
-	    fprintf(stderr,
-		    "vl_Initialize: funny kvno (%d) in ticket, proceeding\n",
-		    ttoken.kvno);
-	    scIndex = 2;
-	}
-    }
-    switch (scIndex) {
-    case 0:
-	sc = rxnull_NewClientSecurityObject();
-	break;
-    case 1:
-	return -1;
-    case 2:
-	sc = rxkad_NewClientSecurityObject(rxkad_clear, &ttoken.sessionKey,
-					   ttoken.kvno, ttoken.ticketLen,
-					   ttoken.ticket);
-    }
-    if (!server) {
-	code = afsconf_GetCellInfo(tdir, NULL, AFSCONF_VLDBSERVICE, &info);
-	if (info.numServers > MAXSERVERS) {
-	    fprintf(stderr,
-		    "vl_Initialize: info.numServers=%d (> MAXSERVERS=%d)\n",
-		    info.numServers, MAXSERVERS);
-	    exit(1);
-	}
-	for (i = 0; i < info.numServers; i++)
-	    serverconns[i] =
-		rx_NewConnection(info.hostAddr[i].sin_addr.s_addr,
-				 info.hostAddr[i].sin_port, USER_SERVICE_ID,
-				 sc, scIndex);
-    } else {
-	serverconns[0] =
-	    rx_NewConnection(server, htons(AFSCONF_VLDBPORT), USER_SERVICE_ID,
-			     sc, scIndex);
-    }
-    code = ubik_ClientInit(serverconns, &cstruct);
-
-    if (code) {
-	fprintf(stderr, "vl_Initialize: ubik client init failed.\n");
-	return code;
-    }
-    return 0;
+    return ugen_ClientInit(auth?0:1, confDir, cellp, 0,
+			  &cstruct, NULL, "vl_Initialize", rxkad_clear, 
+			  MAXSERVERS, AFSCONF_VLDBSERVICE, 50, server,
+			  htons(AFSCONF_VLDBPORT), USER_SERVICE_ID);
 }
 
 /* return host address in network byte order */
 afs_int32
-GetServer(aname)
-     char *aname;
+GetServer(char *aname)
 {
     register struct hostent *th;
     afs_int32 addr;
@@ -277,9 +184,8 @@ GetServer(aname)
 }
 
 
-static
-handleit(as)
-     struct cmd_syndesc *as;
+static int
+handleit(struct cmd_syndesc *as, void *arock)
 {
     register struct cmd_item *ti;
     register afs_int32 code, server = 0, sawserver = 0;
@@ -1123,7 +1029,7 @@ main(argc, argv)
     afs_int32 code;
 
     strcpy(confdir, AFSDIR_CLIENT_ETC_DIRPATH);
-    ts = cmd_CreateSyntax("initcmd", handleit, 0, "initialize the program");
+    ts = cmd_CreateSyntax("initcmd", handleit, NULL, "initialize the program");
     cmd_AddParm(ts, "-cellpath", CMD_LIST, CMD_OPTIONAL,
 		"Cell configuration directory");
     cmd_AddParm(ts, "-server", CMD_LIST, CMD_OPTIONAL,
@@ -1408,8 +1314,9 @@ dump_stats(stats, vital_header)
 {
     int i;
     char strg[30];
+    time_t start_time = stats->start_time;
 
-    strncpy(strg, ctime((time_t *) & stats->start_time), sizeof(strg));
+    strncpy(strg, ctime(&start_time), sizeof(strg));
     strg[strlen(strg) - 1] = 0;
     printf("Dynamic statistics stats (starting time: %s):\n", strg);
     printf("OpcodeName\t# Requests\t# Aborts\n");
