@@ -113,11 +113,16 @@ extern pthread_t vol_glock_holder;
 #define VTRANS_UNLOCK
 #endif /* AFS_PTHREAD_ENV */
 
-typedef enum { fileServer,       /* the fileserver process */
-	       volumeUtility,    /* volserver, or a single volume salvager (non-dafs) */
-	       salvager,         /* standalone whole-partition salvager */
-	       salvageServer,    /* dafs online salvager */
-	       debugUtility      /* fssync-debug or similar utility */
+/**
+ * volume package program type enumeration.
+ */
+typedef enum {
+    fileServer          = 1,    /**< the fileserver process */
+    volumeUtility       = 2,    /**< volserver, or a 
+				 *   single volume salvager (non-dafs) */
+    salvager            = 3,    /**< standalone whole-partition salvager */
+    salvageServer       = 4,    /**< dafs online salvager */
+    debugUtility        = 5     /**< fssync-debug or similar utility */
 } ProgramType;
 extern ProgramType programType;	/* The type of program using the package */
 
@@ -581,7 +586,7 @@ typedef struct Volume {
     VolumeId hashid;		/* Volume number -- for hash table lookup */
     struct volHeader *header;	/* Cached disk data */
     Device device;		/* Unix device for the volume */
-    struct DiskPartition
+    struct DiskPartition64
      *partition;		/* Information about the Unix partition */
     struct vnodeIndex {
 	IHandle_t *handle;	/* Unix inode holding this index */
@@ -615,6 +620,7 @@ typedef struct Volume {
     byte specialStatus;		/* An error code to return on VGetVolume: the
 				 * volume is unavailable for the reason quoted,
 				 * currently VBUSY or VMOVED */
+    afs_uint32 checkoutMode;    /* for volume utilities, mode number for current checkout */
     afs_uint32 updateTime;	/* Time that this volume was put on the updated
 				 * volume list--the list of volumes that will be
 				 * salvaged should the file server crash */
@@ -649,6 +655,7 @@ struct volHeader {
 #define V_vnodeIndex(vp)	((vp)->vnodeIndex)
 #define V_nextVnodeUnique(vp)	((vp)->nextVnodeUnique)
 #define V_linkHandle(vp)	((vp)->linkHandle)
+#define V_checkoutMode(vp)      ((vp)->checkoutMode)
 #ifdef AFS_DEMAND_ATTACH_FS
 #define V_attachState(vp)       ((vp)->attach_state)
 #define V_attachFlags(vp)       ((vp)->attach_flags)
@@ -758,8 +765,8 @@ extern void VSetDiskUsage(void);
 extern void VPrintCacheStats(void);
 extern void VReleaseVnodeFiles_r(Volume * vp);
 extern void VCloseVnodeFiles_r(Volume * vp);
-extern struct DiskPartition *VGetPartition(char *name, int abortp);
-extern struct DiskPartition *VGetPartition_r(char *name, int abortp);
+extern struct DiskPartition64 *VGetPartition(char *name, int abortp);
+extern struct DiskPartition64 *VGetPartition_r(char *name, int abortp);
 extern int VInitVolumePackage(ProgramType pt, afs_uint32 nLargeVnodes,
 			      afs_uint32 nSmallVnodes, int connect, afs_uint32 volcache);
 extern void DiskToVolumeHeader(VolumeHeader_t * h, VolumeDiskHeader_t * dh);
@@ -773,10 +780,10 @@ extern Volume *VPreAttachVolumeByName(Error * ec, char *partition, char *name);
 extern Volume *VPreAttachVolumeByName_r(Error * ec, char *partition, char *name);
 extern Volume *VPreAttachVolumeById_r(Error * ec, char * partition, 
 				      VolId volumeId);
-extern Volume *VPreAttachVolumeByVp_r(Error * ec, struct DiskPartition * partp, 
+extern Volume *VPreAttachVolumeByVp_r(Error * ec, struct DiskPartition64 * partp, 
 				      Volume * vp, VolId volume_id);
 extern Volume *VGetVolumeByVp_r(Error * ec, Volume * vp);
-extern int VShutdownByPartition_r(struct DiskPartition * dp);
+extern int VShutdownByPartition_r(struct DiskPartition64 * dp);
 extern int VShutdownVolume_r(Volume * vp);
 extern int VConnectSALV(void);
 extern int VConnectSALV_r(void);
